# -*- coding: utf-8 -*-
# Generated by Django 1.9.1 on 2016-01-24 01:26
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Badge',
            fields=[
                ('id', models.AutoField(auto_created=True,
                                        primary_key=True,
                                        serialize=False,
                                        verbose_name='ID')),
                ('name', models.TextField()),
                ('code', models.CharField(max_length=100)),
                ('user', models.ForeignKey(
                    on_delete=django.db.models.deletion.CASCADE,
                    to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AlterUniqueTogether(
            name='badge',
            unique_together=set([('user', 'name')]),
        ),
    ]



from __future__ import unicode_literals
from django.conf import settings
from django.db import models


class Badge(models.Model):
    """
    Badges are earned by users for doing nice things.
    They earn the badge when as a `condition` is meet.

    Badges and conditions are set in the badge.json file.  Here we
    only link a user to a badge.
    Badges are uniquely identified by a code.
    A User cannot have the same badge twice.
    """

    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    name = models.TextField()
    code = models.CharField(max_length=100)

    class Meta:
        unique_together = ('user', 'name')



import datetime
from django.conf import settings
from django.core.management import call_command
from django.test import TestCase
from django.contrib.auth.models import User
from autofixture import AutoFixture

from badges.models import Badge
from badges.management.commands.create_badge_triggers import (
    create_trigger, drop_existing_triggers)

from dummy.models import Sketch


class TriggerMixin(object):
    """
    A convenient mixin for setup and teardown
    """
    def setUp(self):
        fixture = AutoFixture(User)
        fixture.create(100)

    def tearDown(self):
        drop_existing_triggers()


class PionnerTest(TriggerMixin, TestCase):
    """
    Check if pionner trigger is efficient
    """

    def test_trigger_create(self):
        """
        Create the pionner trigger, check if it does it's work
        """
        data = {
            "code": "pionner",
            "condition": "age(NEW.date_joined) > interval '1 year'",
            "name": "Pionner",
            "trigger_condition": "update",
            "trigger_table": "auth_user",
            "user_field": "id"
        }
        create_trigger(data)

        self.assertEqual(Badge.objects.count(), 0)
        users = User.objects.all()[:10]
        for user in users:
            user.date_joined = user.date_joined - datetime.timedelta(days=370)
            user.save()
        self.assertEqual(Badge.objects.count(), 10)


class CollectorTest(TriggerMixin, TestCase):
    """
    Check if pionner trigger is efficient
    """

    def test_trigger_create(self):
        """
        Create the collector trigger, check if it does it's work
        """

        data = {
            "code": "collector",
            "condition": "count(id) >= 5 from dummy_sketch where user_id = NEW.user_id",
            "name": "Collector",
            "trigger_condition": "insert",
            "trigger_table": "dummy_sketch",
            "user_field": "user_id"
        }
        create_trigger(data)

        self.assertEqual(Badge.objects.count(), 0)
        users = User.objects.all()[:10]
        for user in users:
            fixture = AutoFixture(
                Sketch,
                field_values={"user": user})
            fixture.create(5)
        self.assertEqual(Badge.objects.count(), 10)


class StarTest(TriggerMixin, TestCase):
    """
    Check if pionner trigger is efficient
    """

    def test_trigger_create(self):
        """
        Create the star trigger, check if it does it's work
        """

        data = {
            "code": "star",
            "condition": "NEW.hit_views >= 1000 ",
            "name": "Star",
            "trigger_condition": "update",
            "trigger_table": "dummy_sketch",
            "user_field": "user_id"
        }
        create_trigger(data)

        self.assertEqual(Badge.objects.count(), 0)
        users = User.objects.all()[:10]
        for user in users:
            fixture = AutoFixture(
                Sketch,
                field_values={"user": user, "hit_views": 0})
            sketch = fixture.create(1)[0]
            sketch.hit_views = 1000
            sketch.save()
        self.assertEqual(Badge.objects.count(), 10)


class TestTrigger(TriggerMixin, TestCase):
    """Here we test the whole story. """

    def test_no_triggers(self):
        """
        Try to create event that would normaly launch badge earning.
        Ensure it's not.
        """
        users = User.objects.all()[:10]
        for user in users:
            user.date_joined = user.date_joined - datetime.timedelta(days=370)
            user.save()
        # no pionner
        self.assertEqual(Badge.objects.count(), 0)

        for user in users:
            fixture = AutoFixture(
                Sketch,
                field_values={"user": user})
            fixture.create(5)
        # no collector
        self.assertEqual(Badge.objects.count(), 0)

        for user in users:
            fixture = AutoFixture(
                Sketch,
                field_values={"user": user, "hit_views": 0})
            sketch = fixture.create(1)[0]
            sketch.hit_views = 1000
            sketch.save()
        # no star
        self.assertEqual(Badge.objects.count(), 0)

    def test_with_trigger_file(self):
        """
        So ok no test but with the json file set in the settings
        """
        call_command("create_badge_triggers")
        # Do not be mean with me, we test only one command ok ?
        users = User.objects.all()[:10]
        for user in users:
            user.date_joined = user.date_joined - datetime.timedelta(days=370)
            user.save()
        # pionners \0/
        self.assertEqual(Badge.objects.count(), 10)

    def test_with_file_parameter(self):
        """ You can call the command with your own file, let's test this"""
        call_command("create_badge_triggers", file=settings.BADGES_FILE)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from os.path import join, dirname, abspath
import sys

PY2 = sys.version_info[0] == 2
ROOT = dirname(abspath(__file__))

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup  # NOQA


def read_relative_file(filename):
    """Returns contents of the given file, whose path is supposed relative
    to this module."""
    with open(join(ROOT, filename)) as f:
        return f.read()

NAME = 'django-pg-badges'
DESCRIPTION = """Create badge for your django user with postgresql triggers"""
REQUIREMENTS = [
    'Django',
    'psycopg2',
]
__VERSION__ = read_relative_file('VERSION').strip()


params = dict(
    name=NAME,
    description=DESCRIPTION,
    packages=['badges'],
    version=__VERSION__,
    long_description=read_relative_file('README.md'),
    author='Yohann Gabory',
    author_email='boblefrag@gmail.com',
    url='https://github.com/boblefrag/django-pg-badges',
    license='MIT License',
    include_package_data=True,
    install_requires=REQUIREMENTS,
    test_requires=["django-autofixture"],
    zip_safe=False,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
)

if __name__ == '__main__':
    setup(**params)



import json
from django.db import connection
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from os.path import dirname, join

"""
Commands used to create or update badges triggers.
"""

DROP_TEMPLATE = """
SELECT proname, relname FROM
    (pg_trigger JOIN pg_class ON tgrelid=pg_class.oid)
JOIN pg_proc ON (tgfoid=pg_proc.oid) WHERE proname LIKE 'badge_%';
"""

class Command(BaseCommand):
    def add_arguments(self, parser):

        parser.add_argument(
            '--file',
            action='store_true',
            dest='file',
            default=None,
            help='Chose a badge file')

    def handle(self, *args, **options):
        if options["file"]:
            fd = options["file"]
        else:
            if hasattr(settings, "BADGES_FILE"):
                fd = settings.BADGES_FILE
            else:
                raise CommandError(
                    "You must set the BADGES_FILE location in your settings"
                )
        with open(fd) as json_file:
            create_triggers(json_file.read())


def validate(badge_dict):
    for key in ["name", "code", "condition",
                "trigger_condition", "trigger_table",
                "user_field"]:
        if not badge_dict.get(key):
            raise ValueError("the badge definition need a {} key".format(key))


def drop_existing_triggers():
    """
    Every badges triggers are prefixed with "badge_"
    So we first get all the existing triggers and delete them.
    One by one... No mercy.
    """
    with connection.cursor() as cur:
        cur.execute(DROP_TEMPLATE)

        for result in cur.fetchall():
            cur.execute("DROP TRIGGER {} ON {}".format(result[0], result[1]))


def create_trigger(badge_dict, check_needed=True):
    """
    Create the underlying triggers for the badge process to work
    """
    if check_needed:
        validate(badge_dict)
    with open(join(dirname(__file__), "trigger.sql")) as sql:
        with connection.cursor() as cur:
            # We do not catch errors here because we want the user
            # beeing able to easily debug with as much information as
            # possible
            cur.execute(sql.read().format(**badge_dict))


def create_triggers(badge_files):
    """
    Create all the triggers in the badges.json file
    """
    # No error catching here. If there is an error in the json file,
    # we want to know as soon as possible.
    badge_dicts = json.loads(badge_files)
    # We check for missing keys in the badge_dicts because we will
    # soon destroy existing badges and we want to be able to revert
    # before old triggers been destroyed
    for badge_dict in badge_dicts:
        validate(badge_dict)
    # If all triggers validated, we can destroy old triggers:
    drop_existing_triggers()
    # and create triggers for each dict we get without futher validation:
    for badge_dict in badge_dicts:
        create_trigger(badge_dict, check_needed=False)
